/**
 * 
 */
package com.platomix.userprofile.handler;

import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.Date;

import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.platomix.userprofile.config.ConfigProperties;
import com.platomix.userprofile.enums.DsEnum;
import com.platomix.userprofile.handler.core.CollectAllTagHandler;
import com.platomix.userprofile.handler.support.ForecastCancelTagManager;
import com.platomix.userprofile.handler.support.UserEventManager;
import com.platomix.userprofile.handler.support.UserTagManager;
import com.platomix.userprofile.handler.support.XxhTagManager;
import com.platomix.userprofile.service.DynamicExecuteService;

import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * @author wookvn
 *
 */
@Slf4j
@Component
public class EntrypointExecute implements ApplicationContextAware{
	
	private ApplicationContext applicationContext;
	
	@Autowired
	private ConfigProperties configProperties;
	
	@Autowired
	private DynamicExecuteService dynamicExecuteService;
	
	//特殊标签，无须每次都生成，当且仅当信息化大网标签数据有新增时，生成一次即可
	static final String longTagHandlerClassName = "xxhLongTagHandler";
	
	//全量运行时的handler
	static final String[] classHandlers=new String[]
			{ "xxhTagHandler", "xxhAppTagHandler", "orderTagHandler", "rmfTagHandler",
					"userOrderStateTagHandler", "orderSumDateTagHandler", "appPreferenceTagHandler",
					"complainTagHandler","forecastCancelTagHandler", "forecastOrderTagHandler"};
	//信息化表相关的handler
//	static final String[] xxhClassHandlers=new String[]
//			{ "xxhTagHandler", "xxhAppTagHandler","appPreferenceTagHandler"};
	
	/**
	 * 执行入口
	 */
	public void startProg() {
		_initDefaultConfigProperties();
		log.info("启动参数：" + JSON.toJSONString(configProperties));
		
		/**
		 * 生成全部标签【默认是上一个月】
		 */
		if ("zt_tag".equalsIgnoreCase(configProperties.getSymbol())) {
			for (String handler : classHandlers) {
				applicationContext.getBean(handler, AbstractTagHandler.class).handle(configProperties.getMonth());
			}
			//生成rfm表 todo
		}
		
		/**
		 * 聚合数据到临时表
		 */
		if("collect_all_tag".equalsIgnoreCase(configProperties.getSymbol())){
			applicationContext.getBean("collectAllTagHandler", AbstractTagHandler.class).handle(configProperties.getMonth());
		}
		
		/**
		 * 临时表数据插入到集群本地表
		 */
		if("join_all".equalsIgnoreCase(configProperties.getSymbol())){
			applicationContext.getBean(FlatTagHandler.class).tempDataToFlatView(configProperties.getMonth());
		}
		
		/**
		 * 执行多个handler，支持多个月【默认上个月】
		 */
		if ("multi_handler".equalsIgnoreCase(configProperties.getSymbol())) {
			if(configProperties.getHandlers() == null || configProperties.getHandlers().isEmpty()) {
				throw new RuntimeException("请传递执行的handler名称，小写字母开头，多个逗号隔开[--profile.handlers]");
			}
			LocalDate monthDate = LocalDate.parse(configProperties.getMonth()+"01", DateTimeFormatter.BASIC_ISO_DATE);
			LocalDate endMonthDate = LocalDate.parse(configProperties.getEndMonth()+"01", DateTimeFormatter.BASIC_ISO_DATE);
			long monthInterval = monthDate.until(endMonthDate, ChronoUnit.MONTHS) + 1;
			for(String classHandler : configProperties.getHandlers()) {
				applicationContext.getBean(classHandler, AbstractTagHandler.class).handleMultiMonth(monthDate, monthInterval);
			}
		}
		
		/**
		 * 聚合，合并多个月的标签数据
		 */
		if("multi_zttag_collect_join".equalsIgnoreCase(configProperties.getSymbol())) {
			//计算日期
			LocalDate monthDate = LocalDate.parse(configProperties.getMonth()+"01", DateTimeFormatter.BASIC_ISO_DATE);
			LocalDate endMonthDate = LocalDate.parse(configProperties.getEndMonth()+"01", DateTimeFormatter.BASIC_ISO_DATE);
			long monthInterval = monthDate.until(endMonthDate, ChronoUnit.MONTHS) + 1;
			//获取handler列表
			String[] _classHandlers = classHandlers;
			if(configProperties.getHandlers() != null && !configProperties.getHandlers().isEmpty()) {
				_classHandlers = configProperties.getHandlers().stream().toArray(String[]::new);
			}
			for(int i = 0; i < monthInterval; i++) {
				String monthStr = DateUtil.format(monthDate.plusMonths(i).atStartOfDay(), "yyyyMM");
				//批量执行多个月份
				for(String classHandler : _classHandlers) {
					//Long中间表只需要跑一次【跑多个月标签数据之前，先跑一次Long中间表】
					if(!longTagHandlerClassName.equalsIgnoreCase(classHandler)) {
						applicationContext.getBean(classHandler, AbstractTagHandler.class).handle(monthStr);
					}
				}
				//聚合标签
				applicationContext.getBean(CollectAllTagHandler.class).handle(monthStr);
				//合并标签
				applicationContext.getBean(FlatTagHandler.class).tempDataToFlatView(monthStr);
			}
		}
		
		/**
		 * 更新mysql中的字典表
		 */
		if ("dic_tag".equalsIgnoreCase(configProperties.getSymbol())) {
			applicationContext.getBean(UserEventManager.class).genEventPropToMysql();
			applicationContext.getBean(UserTagManager.class).genSomeTagToMysql();
		}
		
		/**
		 * 更新etl2->xxh
		 */
		if ("xxh_etl".equalsIgnoreCase(configProperties.getSymbol())) {
			applicationContext.getBean(XxhTagManager.class).xxhFromEtlSp(configProperties.getMonth());
		}
		
		/**
		 * 退订订ding
		 */
		if ("z_cancel_data".equalsIgnoreCase(configProperties.getSymbol())) {
			ForecastCancelTagManager manager = applicationContext.getBean(ForecastCancelTagManager.class);
			manager.tagCancelDataC4(configProperties.getMonth(),-1,true);
			manager.tagCancelDataC4V2(configProperties.getMonth());
		}
		
		/**
		 * 从135 向新的库拷贝
		 */
		if ("copy_table".equalsIgnoreCase(configProperties.getSymbol())) {
			try {
				applicationContext.getBean(XxhTagManager.class).createTables(configProperties.getExt());
			} catch (Exception e) {
				log.info(e.getMessage(), e);
			}
		}

	}
	
	/**
	 * 初始化默认值
	 * 特殊说明：Azkaban所有参数，如果UI不传递参数，默认都是0。需要做特殊判断
	 */
	private void _initDefaultConfigProperties() {
		//[如果month传值，endMonth没有传值，endMonth=month]
		if(!StringUtils.isEmpty(configProperties.getMonth()) && !"0".equals(configProperties.getMonth())
				&& (StringUtils.isEmpty(configProperties.getEndMonth()) || "0".equals(configProperties.getEndMonth()))) {
			configProperties.setEndMonth(configProperties.getMonth());
		}
		//执行月份：默认取上个月
		if(StringUtils.isEmpty(configProperties.getMonth()) || "0".equals(configProperties.getMonth())) {
			configProperties.setMonth(DateUtil.format(DateUtil.offsetMonth(new Date(), -1), "yyyyMM"));
		}
		//结束月份：默认取上个月
		if(StringUtils.isEmpty(configProperties.getEndMonth()) || "0".equals(configProperties.getEndMonth())){
			configProperties.setEndMonth(DateUtil.format(DateUtil.offsetMonth(new Date(), -1), "yyyyMM"));
		}
		//清除默认值0
		if(configProperties.getHandlers() != null && !configProperties.getHandlers().isEmpty()) {
			configProperties.getHandlers().removeIf(f -> "0".equals(f));
		}
		//初始化信息化最新数据的月份
		JSONObject jsonObject = dynamicExecuteService.get(DsEnum.clickhouse, "select max(MONTH_ID) theLatestMonth from XXH_ALL_MONTH");
		configProperties.setXxhLatestMonth(jsonObject.getString("theLatestMonth"));
	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		this.applicationContext = applicationContext;
	}

}
